class Sword extends RocketLauncher;

#EXEC OBJ LOAD FILE=InterfaceContent.utx
#EXEC OBJ LOAD FILE=Generations.utx

var Sound       ParrySound;

var bool canLock;
var Pawn LockedOnTo;
var float lockRange;
var xPlayer xp;

function toggleLock()
{
     if(bLockedOn)//if already locked on, then lock off, else you can search for target
     {
         bLockedOn = false;
         Instigator.Controller.GotoState('PlayerWalking');
     }
     else
         canLock = true;
}

function Tick(float DeltaTime)
{
   local Pawn Other;
    local Vector StartTrace;
    local Rotator Aim;
    local float BestDist, BestAim;

	Super.tick(deltaTime);
    Instigator.PointOfView();
    if (Instigator == None || Instigator.Weapon != self)
        return;

	if ( Instigator.Role < ROLE_Authority )
		return;

    if ( !Instigator.IsHumanControlled() )
        return;

        StartTrace = Instigator.Location + Instigator.EyePosition();
        Aim = Instigator.GetViewRotation();

        BestAim = LockAim;

        //this next line always returns none in netgames, I assume native functions don't run on autonomous proxies?
        Other = Instigator.Controller.PickTarget(BestAim, BestDist, Vector(Aim), StartTrace, SeekRange);
        if(Other!=none && !bLockedOn)
        {
         lockedOnTo = Other;
        }


        if ( CanLockOnTo(Other) )
        {
            if (Other == SeekTarget)
            {
                LockTime += SeekCheckFreq;
                if (!bLockedOn && LockTime >= LockRequiredTime)
                {
                    if(canLock)
                    {
                        bLockedOn = true;

                        Instigator.Controller.GotoState('PlayerLockedOn');
                        PlayerController(Instigator.Controller).ClientPlaySound(Sound'WeaponSounds.LockOn');
                    }
                 }
            }
            else
            {
                SeekTarget = Other;
                LockTime = 0.0;
            }
        }


        canLock = false;

        SeekCheckTime = Level.TimeSeconds + SeekCheckFreq;
        if(lockedOnTo != None)
        {
              if(vSize(lockedOnTo.Location - (Instigator.Location + Instigator.EyePosition())) > lockRange)
              {
                  bLockedOn = false;
                  Instigator.Controller.GotoState('PlayerWalking');
                  other = none;
              }

              if(bLockedOn == true)
              {
               Deify_xPlayer(Instigator.Controller).lockOnTo(lockedOnTo);
              }

              if(lockedOnTo.Health <= 0)
              {
                  bLockedOn = false;
                  Instigator.Controller.GotoState('PlayerWalking');
                  other = none;
              }
         }

}

simulated function bool CanLockOnTo(Actor Other)
{
    local Pawn P;

    P = Pawn(Other);

    if (P == None || P == Instigator || !P.bProjTarget)
        return false;

	if ( GENTroopPawn(P) != none && GENTroopPawn(P).TeamNum == Instigator.PlayerReplicationInfo.Team.TeamIndex)
		return false;

	if ( P.PlayerReplicationInfo != none && Instigator.PlayerReplicationInfo.Team.TeamIndex == P.PlayerReplicationInfo.Team.TeamIndex )
		return false;

	if (vSize(other.Location - (Instigator.Location + Instigator.EyePosition())) > lockRange)
		return false;

    return true;
}

simulated state AnimateLoad
{}


simulated function DoAutoSwitch()
{
}

simulated function vector CenteredEffectStart()
{
    local Vector X,Y,Z;

    GetViewAxes(X, Y, Z);
    return (Instigator.Location +
        Instigator.CalcDrawOffset(self) +
        EffectOffset.X * X +
        EffectOffset.Z * Z);
}

simulated event RenderOverlays( Canvas Canvas )
{
    local int m;

    if ((Hand < -1.0) || (Hand > 1.0))
    {
		for (m = 0; m < NUM_FIRE_MODES; m++)
		{
			if (FireMode[m] != None)
			{
				FireMode[m].DrawMuzzleFlash(Canvas);
			}
		}
	}
    Super.RenderOverlays(Canvas);
}

// AI Interface
function GiveTo(Pawn Other, optional Pickup Pickup)
{
	Super.GiveTo(Other, Pickup);

	if ( Bot(Other.Controller) != None )
		Bot(Other.Controller).bHasImpactHammer = true;
}

function bool CanAttack(Actor Other)
{
	if (VSize(Other.Location - Instigator.Location) > 300)
		return false;
	return true;
}

function byte BestMode()
{
	local bot B;

	B = Bot(Instigator.Controller);
	if ( (B == None) || (B.Enemy == None) )
		return 0;

	return 0;
}

// super desireable for bot waiting to impact jump
function float GetAIRating()
{
	local Bot B;
	local float EnemyDist;

	B = Bot(Instigator.Controller);
	if ( B == None )
		return AIRating;


	if ( B.Enemy == None )
		return AIRating;

	EnemyDist = VSize(B.Enemy.Location - Instigator.Location);
	if ( B.Stopped() && (EnemyDist > 400) )
		return 0.1;
}

// End AI interface

function AdjustPlayerDamage( out int Damage, Pawn InstigatedBy, Vector HitLocation,
						         out Vector Momentum, class<DamageType> DamageType)
{
	if (FireMode[1].bIsFiring && (DamageType == class'SwordDamType' || DamageType == class'SkaarjPack.MeleeDamage') && CheckParry(HitLocation))
	{
		Damage = 0;
		PlaySound(ParrySound, SLOT_None);
	}
}

simulated function float AmmoStatus(optional int Mode) // returns float value for ammo amount
{
	if ( Instigator == None || Instigator.Weapon != self )
	{
		if ( (FireMode[1].TimerInterval != 0.f) && (FireMode[1].NextTimerPop < Level.TimeSeconds) )
		{
			FireMode[1].Timer();
  			if ( FireMode[1].bTimerLoop )
				FireMode[1].NextTimerPop = FireMode[1].NextTimerPop + FireMode[1].TimerInterval;
			else
				FireMode[1].TimerInterval = 0.f;
		}
	}
	return Super.AmmoStatus(Mode);
}


function bool CheckParry( Vector HitLocation)
{
    local Vector HitDir;
    local Vector FaceDir;

    FaceDir = Vector(Instigator.Controller.Rotation);
    HitDir = Normal(Instigator.Location - HitLocation + Vect(0,0,8));

    if ( FaceDir dot HitDir < -0.8 )
        return true;

    return false;
}

function float SuggestAttackStyle()
{
    return 0.8;
}

function float SuggestDefenseStyle()
{
    return -0.8;
}

function bool RecommendRangedAttack()
{
	return false;
}

function setDmgMultiplyer(float mult)
{
	if (SwordFire(FireMode[0]) != none)
	{
		SwordFire(FireMode[0]).damage = SwordFire(FireMode[0]).default.Damage * mult;
	}
}

defaultproperties
{
	HighDetailOverlay=Material'UT2004Weapons.WeaponSpecMap2'
    IconMaterial=Material'Generations.SwordIcon'
    IconCoords=(X1=0,Y1=0,X2=72,Y2=38)

	//bShowChargingBar=true
    bCanThrow=false
    InventoryGroup=1
    BobDamping=2.2
    bMeleeWeapon=true
    ParrySound=Sound'WeaponSounds.ShieldGun.ShieldReflection'
    DrawScale=1.0
    DisplayFOV=110
    EffectOffset=(X=100,Y=25,Z=-3)
    PlayerViewOffset=(X=-5,Y=-3,Z=10)
    SmallViewOffset=(X=2,Y=0,Z=-1.5)
    PlayerViewPivot=(Pitch=0,Roll=0,Yaw=38768)

    SelectSound=Sound'WeaponSounds.ShieldGun_change'
	SelectForce="ShieldGun_change"

	AIRating=0.7
	CurrentRating=0.7
    HudColor=(r=255,g=188,b=121,a=255)

	Priority=8
	CustomCrosshair=6
	CustomCrosshairTextureName="Crosshairs.Hud.Crosshair_Pointer"
	CustomCrosshairScale=1.0

	CenteredOffsetY=-9.0
	CenteredRoll=1000

     lockRange=800.000000
     LockRequiredTime=0.000000
     FireModeClass(0)=Class'GTDeify.SwordFire'
     FireModeClass(1)=Class'GTDeify.SwordParryFire'
     Description="Swing for massive damage."
     CustomCrossHairColor=(B=20,G=20,R=200)
     PickupClass=Class'GTDeify.SwordPickup'
     AttachmentClass=Class'GTDeify.SwordAttatchment'
     ItemName="Sword"
     Mesh=SkeletalMesh'Sword.sword_1stPerson'
     OldMesh=None

    IdleAnim=Idle
    RestAnim=Rest
    AimAnim=Aim
    RunAnim=Run
    SelectAnim=Select
    PutDownAnim=PutDown

}
