//-----------------------------------------------------------
//
//-----------------------------------------------------------

class GENGameInfo extends xTeamGame;

var array<GENPowerCore>         Bases;
var int                  HQTeam[2];

// GameUMenuType - This menu is displayed to players when they die - it should contain the map for choosing where to respawn at

struct PowerLinkSetup
{
	var name BaseNode;
	var array<name> LinkedNodes;
};

var array<GENPowerLinkOfficialSetup> OfficialPowerLinkSetups;
var array<GENPowerLinkCustomSetup> CustomPowerLinkSetups;
var array<GENPowerLink> PowerLinks;

var config int OvertimeCoreDrainPerSec;
var config bool bRandSetupAfterReset;	//choose a random link setup to play after a reset
var config bool bSwapSidesAfterReset;
var bool        bSidesAreSwitched;

var bool bCurrentSetupIsOfficial;
var string CurrentSetupName;

// localized PlayInfo descriptions & extra info
const GENPROPNUM = 4;
var localized string GENPropsDisplayText[GENPROPNUM];
var localized string GENPropDescText[GENPROPNUM];

var string botClassName[3];
var config bool bUseCustomModels;


var(LoadingHints) private localized array<string> GENHints;

function bool AllowTransloc()
{
	return false;
}

static function bool NeverAllowTransloc()
{
	return true;
}

static function bool AllowMutator( string MutatorClassName )
{
	if ( MutatorClassName == "" )
		return false;

	if ( MutatorClassName ~= "xGame.MutInstagib" )
		return false;
	if ( MutatorClassName ~= "xGame.MutZoomInstagib" )
		return false;

	return super.AllowMutator(MutatorClassName);
}

function Bot SpawnBot(optional string botName)
{
local Bot newBot;
local RosterEntry newEntry;
local UnrealTeamInfo botTeam;
local int chosenClass;

    botTeam = GetBotTeam();
    newEntry = botTeam.ChooseBotClass(botName);

    chosenClass = rand(3);
    switch ( chosenClass )
    {
    	case 0:
    		newEntry.PawnClassName = "GTDeify.Deify_xPawn_Berserker";
    		break;
    	case 1:
    		newEntry.PawnClassName = "GTDeify.Deify_xPawn_Destructor";
    		break;
    	case 2:
    		newEntry.PawnClassName = "GTDeify.Deify_xPawn_Summoner";
    		break;
    }
	
    newBot = Spawn(class<xBot>(DynamicLoadObject("GTDeify.Deify_xBot", class'Class')));

    if ( newBot != None )
        InitializeBot(newBot,botTeam,newEntry);
    return newBot;
}

event SetGrammar()
{
	//LoadSRGrammar("");
}

event PostBeginPlay()
{
	SetupLinks();
	Super.PostBeginPlay();
}

function bool ApplyOrder( PlayerController Sender, int RecipientID, int OrderID )
{
	local controller P;

	if ( OrderID == 299 )
	{
		for ( P=Level.ControllerList; P!= None; P=P.NextController )
		{
			if ( (Bot(P) != None) && (Bot(P).Pawn != None) && (P.PlayerReplicationInfo != None) && (P.PlayerReplicationInfo.Team == Sender.PlayerReplicationInfo.Team)
				&& ((RecipientID == -1) || (RecipientID == P.PlayerReplicationInfo.TeamID)) )
			{
				Bot(P).GetOutOfVehicle();
				if ( RecipientID == P.PlayerReplicationInfo.TeamID )
					break;
			}
		}
		return true;
	}

	return Super.ApplyOrder(Sender,RecipientID,OrderID);
}

function int ParseOrder(string OrderString)
{
	if ( OrderString == "GET OUT" )
		return 299;
	return Super.ParseOrder(OrderString);
}

static function PrecacheGameAnnouncements(AnnouncerVoice V, bool bRewardSounds)
{
	Super.PrecacheGameAnnouncements(V,bRewardSounds);
	if ( !bRewardSounds )
	{
		V.PrecacheSound('Red_powernode_destroyed');
		V.PrecacheSound('Blue_powernode_destroyed');
		V.PrecacheSound('Red_Powercore_under_attack');
		V.PrecacheSound('Blue_Powercore_under_attack');
		V.PrecacheSound('NewRoundIn');
		V.PrecacheSound('Red_Powercore_destroyed');
		V.PrecacheSound('Blue_Powercore_destroyed');
		V.PrecacheSound('Red_Powercore_critical');
		V.PrecacheSound('Blue_Powercore_critical');
		V.PrecacheSound('Red_Powercore_vulnerable');
		V.PrecacheSound('Blue_Powercore_vulnerable');
		V.PrecacheSound('Red_Powernode_under_construction');
		V.PrecacheSound('Blue_Powernode_under_construction');
		V.PrecacheSound('Red_powercore_damaged');
		V.PrecacheSound('Blue_powercore_damaged');
		V.PrecacheSound('Red_powernode_isolated');
		V.PrecacheSound('Blue_powernode_isolated');
	}
}

function SetInitialState()
{
    Super.SetInitialState();

    SetCoreDistance(Bases[HQTeam[0]], 0, 0);
    SetCoreDistance(Bases[HQTeam[1]], 1, 0);

    UpdateLinks();
}

event InitGame(string Options, out string Error)
{
	local string InOpt, MapName;
	local GENPowerLinkOfficialSetup P;
	local int x;
	local NavigationPoint N;
	local array<string> CustomPowerLinkSetupNames;

    Super.InitGame(Options, Error);
	x = 0;
	for(N = Level.NavigationPointList; N != None; N = N.NextNavigationPoint)
	{
		// Reset all PlayerStarts so that the Bases can determine their team and Pre-Rate them
		if (N.IsA('PlayerStart'))
			PlayerStart(N).bEnabled = False;
		// Register all the Bases
		if (N.IsA('GENPowerCore'))
		{
			Bases[x] = GENPowerCore(N);
			Bases[x].NotifyUpdateLinks = UpdateLinks;
			Bases[x].OnCoreDestroyed = MainCoreDestroyed;

			if (Bases[x].bFinalCore)
				HQTeam[Bases[x].DefenderTeamIndex] = x;
			x++;
		}
	}

	if (Bases.Length == 0)
	{
		log("Generations: Level doesn't have any PowerCores!",'Error');
		return;
	}

	MapName = Left(string(Level), InStr(string(Level), "."));


	OfficialPowerLinkSetups.length = 0;
	foreach AllActors(class'GENPowerLinkOfficialSetup', P)
		{OfficialPowerLinkSetups[OfficialPowerLinkSetups.length] = P;}

	//For PerObjectConfig objects, the Outer overrides the default config file name.
	//This means by giving the Level as the Outer, the .ini name will be the same as the map name,
	//which is really cool because it makes it easy to trade custom setups - just copy that map's .ini
	CustomPowerLinkSetupNames = class'GENPowerLinkCustomSetup'.static.GetPerObjectNames(MapName);
	for (x = 0; x < CustomPowerLinkSetupNames.length; x++)
		CustomPowerLinkSetups[CustomPowerLinkSetups.length] = new(Level, CustomPowerLinkSetupNames[x]) class'GENPowerLinkCustomSetup';

	InOpt = ParseOption(Options, "bRandSetupAfterReset");
	if (InOpt != "")
		bRandSetupAfterReset = bool(InOpt);

	InOpt = ParseOption(Options, "LinkSetup");
	if (InOpt ~= "Random")
	{
		FindLinkSetup();
		return;
	}
	if (InOpt == "" || !SetPowerLinkSetup(InOpt, None))
	{
		if (!SetPowerLinkSetup("Default", None))
			FindLinkSetup();
	}

	// Big hack.. you shouldn't break scope this way :)

 	InOpt = ParseOption(Options,"BonusVehicles");
	if ( bool(InOpt) )
	{
		AddMutator("OnslaughtBP.MutBonusVehicles");
	}

}

function bool SetupExists(string SetupName)
{
	local int x;

	for (x = 0; x < OfficialPowerLinkSetups.length; x++)
		if (OfficialPowerLinkSetups[x].SetupName ~= SetupName)
			return true;

	for (x = 0; x < CustomPowerLinkSetups.length; x++)
		if (string(CustomPowerLinkSetups[x].Name) ~= SetupName)
			return true;

	return false;
}

function bool SetPowerLinkSetup(string SetupName, PlayerController Caller)
{
	local int x, y, z, i;
	local array<GENPowerCore> Traveled;

    SetupName = Repl(SetupName, " ", "_");

	for (x = 0; x < OfficialPowerLinkSetups.length; x++)
		if (OfficialPowerLinkSetups[x].SetupName ~= SetupName)
		{
			//activate this setup
			for (y = 0; y < OfficialPowerLinkSetups[x].LinkSetups.length; y++)
				for (z = 0; z < Bases.length; z++)
					if (Bases[z].Name == OfficialPowerLinkSetups[x].LinkSetups[y].BaseNode)
						Bases[z].LinkedNodes = OfficialPowerLinkSetups[x].LinkSetups[y].LinkedNodes;
			ResetPowerLinks();
			Log("Using Official Link Setup: "$SetupName);
			CurrentSetupName = SetupName;
			bCurrentSetupIsOfficial = true;
			if (Caller != None)
				Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 2);
			return true;
		}

	for (x = 0; x < CustomPowerLinkSetups.length; x++)
		if (string(CustomPowerLinkSetups[x].Name) ~= SetupName)
		{
			//activate this setup
			for (y = 0; y < CustomPowerLinkSetups[x].LinkSetups.length; y++)
				for (z = 0; z < Bases.length; z++)
					if (Bases[z].Name == CustomPowerLinkSetups[x].LinkSetups[y].BaseNode)
						Bases[z].LinkedNodes = CustomPowerLinkSetups[x].LinkSetups[y].LinkedNodes;
			ResetPowerLinks();
			if (PowerLinks.length > 0 && !ExistsLinkPathTo(Bases[HQTeam[0]], Bases[HQTeam[1]], Traveled))
			{
				//trying to load an invalid setup, revert to previous one
				CustomPowerLinkSetups.Remove(x, 1);
				if (Caller != None)
					Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 8);
				else
					Log("Failed to load Custom Link Setup"@SetupName$": There is not a complete path between the PowerCores.");
				if (CurrentSetupName != "" && CurrentSetupName != SetupName && SetPowerLinkSetup(CurrentSetupName, None))
					return true;
				if (OfficialPowerLinkSetups.Length > 0)
				{
					FindLinkSetup();
					return true;
				}
				Log("Can't find a better link setup to load, so using"@SetupName@" anyway and forcing a connection between the PowerCores.");
				AddLink(Bases[HQTeam[0]], Bases[HQTeam[1]]);
				UpdateLinks();
				for (i = 0; i < GameReplicationInfo.PRIArray.Length; i++)
					if (PlayerController(GameReplicationInfo.PRIArray[i].Owner) != None && GENPlayerReplicationInfo(GameReplicationInfo.PRIArray[i]) != None)
						GENPlayerReplicationInfo(GameReplicationInfo.PRIArray[i]).ClientReceivePowerLink(Bases[HQTeam[0]], Bases[HQTeam[1]]);
				return true;
			}
			Log("Using Custom Link Setup: "$SetupName);
			CurrentSetupName = SetupName;
			bCurrentSetupIsOfficial = false;
			if (Caller != None)
				Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 2);
			return true;
		}

	if (Caller != None)
		Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 3);
	return false;
}

function ClearLinks()
{
	local int i;

	for ( i = 0; i < PowerLinks.Length; i++ )
		if ( PowerLinks[i] != None )
			PowerLinks[i].Destroy();

	PowerLinks.Remove(0, PowerLinks.Length);
}

function ResetPowerLinks()
{
	local int x;
	local GENPlayerReplicationInfo PRI;
	if ( PowerLinks.Length == 0 )
		return;

	ClearLinks();

	for ( x = 0; x < Bases.Length; x++ )
		Bases[x].ResetLinks();

	SetupLinks();

	if (GameReplicationInfo != None)
	{
		for (x = 0; x < GameReplicationInfo.PRIArray.length; x++)
		{
			PRI = GENPlayerReplicationInfo(GameReplicationInfo.PRIArray[x]);
			if ( PRI == None || PlayerController(PRI.Owner) == None )
				continue;

			PRI.ClientPrepareToReceivePowerLinks();
		}
	}
	Bases[0].reTier();
}

//Save a powercore link setup. Returns true if it successfully created a NEW setup and neither failed nor overwrote an old setup
function bool SavePowerLinkSetup(string SetupName, PlayerController Caller)
{
	local int x, y, z;
	local PowerLinkSetup BlankSetup;
	local string MapName;

	MapName = Left(string(Level), InStr(string(Level), "."));
	SetupName = Repl(SetupName, " ", "_");

	for (x = 0; x < OfficialPowerLinkSetups.length; x++)
		if (OfficialPowerLinkSetups[x].SetupName ~= SetupName)
		{
			Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 1);
			return false;
		}

	for (x = 0; x < CustomPowerLinkSetups.length; x++)
		if (string(CustomPowerLinkSetups[x].Name) ~= SetupName)
		{
			//overwrite
			CustomPowerLinkSetups[x].LinkSetups.length = 0;
			for (y = 0; y < Bases.length; y++)
			{
				CustomPowerLinkSetups[x].LinkSetups[y] = BlankSetup;
				CustomPowerLinkSetups[x].LinkSetups[y].BaseNode = Bases[y].Name;
				for (z = 0; z < Bases[y].PowerLinks.length; z++)
					CustomPowerLinkSetups[x].LinkSetups[y].LinkedNodes[z] = Bases[y].PowerLinks[z].Name;
			}
			CustomPowerLinkSetups[x].SaveConfig();
			Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 0);
			return false;
		}

	//save new
	x = CustomPowerLinkSetups.length;
	CustomPowerLinkSetups.length = CustomPowerLinkSetups.length + 1;
	CustomPowerLinkSetups[x] = new(Level, SetupName) class'GENPowerLinkCustomSetup';
	CustomPowerLinkSetups[x].LinkSetups.length = 0;
	for (y = 0; y < Bases.length; y++)
	{
		CustomPowerLinkSetups[x].LinkSetups[y] = BlankSetup;
		CustomPowerLinkSetups[x].LinkSetups[y].BaseNode = Bases[y].Name;
		for (z = 0; z < Bases[y].PowerLinks.length; z++)
			CustomPowerLinkSetups[x].LinkSetups[y].LinkedNodes[z] = Bases[y].PowerLinks[z].Name;
	}
	CustomPowerLinkSetups[x].SaveConfig();
	Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 0);

	return true;
}

//Deletes a powercore link setup. Returns true if successful, false if not
function bool DeletePowerLinkSetup(string SetupName, PlayerController Caller)
{
	local int x;
	local string MapName;

	MapName = Left(string(Level), InStr(string(Level), "."));
	SetupName = Repl(SetupName, " ", "_");

	for (x = 0; x < OfficialPowerLinkSetups.length; x++)
		if (OfficialPowerLinkSetups[x].SetupName ~= SetupName)
		{
			Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 5);
			return false;
		}

	for (x = 0; x < CustomPowerLinkSetups.length; x++)
		if (string(CustomPowerLinkSetups[x].Name) ~= SetupName)
		{
			CustomPowerLinkSetups[x].ClearConfig();
			CustomPowerLinkSetups.Remove(x, 1);
			Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 4);
			return true;
		}

	Caller.ReceiveLocalizedMessage(class'GENLinkDesignMessage', 6);
	return false;
}

function FindLinkSetup(optional string IgnoredSetup)
{
	local int i, Sanity, TotalNumSetups;
	local string NewSetupName;

	TotalNumSetups = OfficialPowerLinkSetups.length + CustomPowerLinkSetups.length;

	if (TotalNumSetups == 0)
		Log("Using default Link Setup");
	else if (TotalNumSetups == 1 && CurrentSetupName != "")
		SetPowerLinkSetup(CurrentSetupName, None);
	else
	{
		do
		{
			i = Rand(TotalNumSetups);
			if (i < OfficialPowerLinkSetups.length)
				NewSetupName = OfficialPowerLinkSetups[i].SetupName;
			else
				NewSetupName = string(CustomPowerLinkSetups[i-OfficialPowerLinkSetups.length].Name);
			Sanity++;
		} until (NewSetupName != IgnoredSetup || Sanity > 1000)

		SetPowerLinkSetup(NewSetupName, None);
	}
}

static simulated function bool ExistsLinkPathTo(GENPowerCore Start, GENPowerCore Destination, out array<GENPowerCore> Traveled)
{
	local int i, j;
	local bool bSkip;

	if (Start == None || Destination == None)
		return false;

	Traveled[Traveled.length] = Start;

	for (i = 0; i < Start.PowerLinks.Length; i++)
	{
		if (Start.PowerLinks[i] == Destination)
			return true;
		bSkip = false;
		for (j = 0; j < Traveled.length; j++)
		{
			if (Traveled[j] == Start.PowerLinks[i])
			{
				bSkip = true;
				break;
			}
		}
		if (!bSkip && ExistsLinkPathTo(Start.PowerLinks[i], Destination, Traveled))
			return true;
	}

	return false;
}

function SetCoreDistance(GENPowerCore PC, byte T, byte Distance)
{
    local int i;

    PC.FinalCoreDistance[T] = Distance;

    for (i=0; i<PC.PowerLinks.Length; i++)
    {
        if (PC.PowerLinks[i].FinalCoreDistance[T] > Distance + 1)
            SetCoreDistance(PC.PowerLinks[i], T, Distance + 1);
    }
}

function UpdateLinks()
{
    local int i, j;
    local bool bRedCoreWasVulnerable, bBlueCoreWasVulnerable;

    if (!bScriptInitialized)	//Bases haven't done PostBeginPlay() yet
        return;

//	log(Name@"UpdateLinks");
    bRedCoreWasVulnerable = Bases[HQTeam[0]].bPoweredByBlue;
    bBlueCoreWasVulnerable = Bases[HQTeam[1]].bPoweredByRed;

    UpdateSeveredLinks();

    for (i=0; i<Bases.Length; i++)
    {
        Bases[i].bPoweredByRed = False;
        Bases[i].bPoweredByBlue = False;
        for (j=0; j<Bases[i].PowerLinks.Length; j++)
            if (Bases[i].PowerLinks[j].CoreStage == 0 && !Bases[i].PowerLinks[j].bSevered)
            {
                if (Bases[i].PowerLinks[j].DefenderTeamIndex == 0)
                    Bases[i].bPoweredByRed = True;
                else if (Bases[i].PowerLinks[j].DefenderTeamIndex == 1)
                    Bases[i].bPoweredByBlue = True;
            }

	if (Bases[i].CoreStage == 4 && (Bases[i].bPoweredByRed || Bases[i].bPoweredByBlue))
		Bases[i].CheckTouching();
        Bases[i].CheckShield();
    }

    if (!bRedCoreWasVulnerable && Bases[HQTeam[0]].bPoweredByBlue)
    	BroadcastLocalizedMessage(class'GENGenerationsMessage', 20);
    if (!bBlueCoreWasVulnerable && Bases[HQTeam[1]].bPoweredByRed)
    	BroadcastLocalizedMessage(class'GENGenerationsMessage', 21);
}

function UpdateSeveredLinks()
{
    local int i;
    local array<byte> WasSevered;

    for (i = 0; i < Bases.Length; i++)
    {
        WasSevered[i] = byte(Bases[i].bSevered);
        Bases[i].bSevered = True;
    }

    Sever(Bases[HQTeam[0]], 0);
    Sever(Bases[HQTeam[1]], 1);

    for (i = 0; i < Bases.Length; i++)
        if (!Bases[i].bDisabled && Bases[i].CoreStage != 4)
        {
            if (Bases[i].bSevered && WasSevered[i] == 0)
		Bases[i].Sever();
	    else if (!Bases[i].bSevered && WasSevered[i] == 1)
	    	Bases[i].UnSever();
	}
}

function Sever(GENPowerCore PC, int TeamIndex)
{
    local int i;
    PC.bSevered = False;
    if (PC.CoreStage != 0)
        return;
    for (i = 0; i < PC.PowerLinks.Length; i++)
        if (PC.PowerLinks[i].bSevered && PC.PowerLinks[i].DefenderTeamIndex == TeamIndex)
            Sever(PC.PowerLinks[i], TeamIndex);
}

function Reset()
{
	local GENPowerCore C;
	bOverTime = false;
	bOverTimeBroadcast = false;
	foreach AllActors(class'GENPowerCore', C)
    	C.setBotDest(C);

}

function StartMatch()
{
	local array<GENPowerCore> Traveled;
	local int i;

	//check if the link setup is valid (in case host/admin edited it before game start)
	if (Level.NetMode != NM_Standalone && !ExistsLinkPathTo(Bases[HQTeam[0]], Bases[HQTeam[1]], Traveled))
	{
		//trying to load an invalid setup, revert to previous one
		Log("Current link setup is invalid: There is not a complete path between the Bases.");
		if (CurrentSetupName == "" || !SetPowerLinkSetup(CurrentSetupName, None))
		{
			if (OfficialPowerLinkSetups.length > 0)
				FindLinkSetup();
			else
			{
				Log("Can't find a better link setup to load, so forcing a connection between the Bases.");
				AddLink(Bases[HQTeam[0]], Bases[HQTeam[1]]);
				UpdateLinks();
				for (i = 0; i < GameReplicationInfo.PRIArray.Length; i++)
					if (PlayerController(GameReplicationInfo.PRIArray[i].Owner) != None && GENPlayerReplicationInfo(GameReplicationInfo.PRIArray[i]) != None)
						GENPlayerReplicationInfo(GameReplicationInfo.PRIArray[i]).ClientReceivePowerLink(Bases[HQTeam[0]], Bases[HQTeam[1]]);
			}
		}
	}

	Super.StartMatch();
}

State MatchInProgress
{
	event Timer()
	{
        local Controller C;
    	local int i, TeamNodes[2], TotalNodes;
    	local Actor A;
    	local bool bOldUnderAttack;

        for(i=0;i<Bases.Length;i++)
        {
        	if ( Level.TimeSeconds > Bases[i].LastAttackExpirationTime )
        	{
	            bOldUnderAttack = Bases[i].bUnderAttack;
	            Bases[i].bUnderAttack = (Level.TimeSeconds - Bases[i].LastAttackTime < Bases[i].LastAttackExpirationTime);
	            if (bOldUnderAttack != Bases[i].bUnderAttack)
	                Bases[i].UnderAttackChange();
	        }

            if (!Bases[i].bFinalCore && Bases[i].CoreStage != 255)
    	    {
    	    	if (Bases[i].CoreStage == 0 && Bases[i].DefenderTeamIndex < 2)
    	            	TeamNodes[Bases[i].DefenderTeamIndex]++;
    	        TotalNodes++;
    	    }
        }

        if (bOverTime)
        {
        	Bases[HQTeam[0]].TakeDamage(OvertimeCoreDrainPerSec - OvertimeCoreDrainPerSec * (float(TeamNodes[0]) / TotalNodes), None, vect(0,0,0), vect(0,0,0), None);
        	Bases[HQTeam[1]].TakeDamage(OvertimeCoreDrainPerSec - OvertimeCoreDrainPerSec * (float(TeamNodes[1]) / TotalNodes), None, vect(0,0,0), vect(0,0,0), None);
        }

        Super.Timer(); // Need this!!!

        if (ResetCountDown > 0)
        {
            ResetCountDown--;
            if ( ResetCountDown < 3 )
            {
				// blow up all redeemer guided warheads
				for ( C=Level.ControllerList; C!=None; C=C.NextController )
					if ( (C.Pawn != None) && C.Pawn.IsA('RedeemerWarhead') )
						C.Pawn.Fire(1);

			}
            if ( ResetCountDown == 8 )
            {
                for ( C = Level.ControllerList; C != None; C = C.NextController )
                    if ( PlayerController(C) != None )
                        PlayerController(C).PlayStatusAnnouncement('NewRoundIn',1,true);
            }
            else if ( (ResetCountDown > 1) && (ResetCountDown < 7) )
                BroadcastLocalizedMessage(class'TimerMessage', ResetCountDown-1);
            else if (ResetCountDown == 1)
            {
                Log("Reset Generations.");

            	for (C = Level.ControllerList; C != None; C = C.NextController)
            	{
                    if (PlayerController(C) != None)
                        PlayerController(C).ClientReset();
                    C.Reset();
                }

            	foreach AllActors(class'Actor', A)
            		if (!A.IsA('Controller') && !A.IsA('GENPowerCore') && !A.IsA('TeamInfo'))
            			A.Reset();

            	//we reset Bases after everything else because their reset might cause
            	//vehiclefactories, etc to activate and spawn new actors that we don't want to affect
            	for (i = 0; i < Bases.length; i++)
            	{
                    if (bSwapSidesAfterReset && Bases[i].bFinalCore)
                    {
                        if (bSidesAreSwitched)
                            Bases[i].DefenderTeamIndex = Bases[i].default.DefenderTeamIndex;
                        else
                            Bases[i].DefenderTeamIndex = abs(Bases[i].default.DefenderTeamIndex - 1);

                        HQTeam[Bases[i].DefenderTeamIndex] = i;
                    }
                    else
                        Bases[i].DefenderTeamIndex = Bases[i].default.DefenderTeamIndex;

                    Bases[i].FinalCoreDistance[0] = 255;
                    Bases[i].FinalCoreDistance[1] = 255;

            		Bases[i].Reset();
            	}

            	if (bSwapSidesAfterReset)
            	{
                    SetCoreDistance(Bases[HQTeam[0]], 0, 0);
                    SetCoreDistance(Bases[HQTeam[1]], 1, 0);
                    bSidesAreSwitched = !bSidesAreSwitched;
                }

        		//possibly choose a new link setup
            	if (bRandSetupAfterReset && (OfficialPowerLinkSetups.length + CustomPowerLinkSetups.length) > 1)
            		FindLinkSetup(CurrentSetupName);
            	else
            		UpdateLinks();

                FindNewObjectives(None); //update AI

            	//now respawn all the players
            	for (C = Level.ControllerList; C != None; C = C.nextController)
            		if (C.PlayerReplicationInfo != None && !C.PlayerReplicationInfo.bOnlySpectator)
	            		RestartPlayer(C);

            	//reset timelimit
            	RemainingTime = 60 * TimeLimit;
            	GameReplicationInfo.RemainingMinute = RemainingTime;

                ResetCountDown = 0;
           }
        }
    }
}

function MainCoreDestroyed(byte T)
{
    local Controller C;
    local PlayerController PC;
    local int Score;

    if (bOverTime)
    	Score = 1;
    else
    	Score = 2;

    if (T == 1)
    {
        BroadcastLocalizedMessage( class'GENGenerationsMessage', 0);
        TeamScoreEvent(0, Score, "enemy_core_destroyed");
        Teams[0].Score += Score;
 		Teams[0].NetUpdateTime = Level.TimeSeconds - 1;
       CheckScore(Bases[HQTeam[1]].LastDamagedBy);
    }
    else
    {
        BroadcastLocalizedMessage( class'GENGenerationsMessage', 1);
        TeamScoreEvent(1, Score, "enemy_core_destroyed");
        Teams[1].Score += Score;
		Teams[1].NetUpdateTime = Level.TimeSeconds - 1;
        CheckScore(Bases[HQTeam[0]].LastDamagedBy);
    }

    //round has ended
    foreach AllActors(class'Controller',C)// (C = Level.ControllerList; C != None; C = C.NextController)
    {
    	PC = PlayerController(C);
    	if (PC != None)
    	{
    		PC.ClientSetBehindView(true);
    		PC.ClientSetViewTarget(Bases[HQTeam[T]]);
    		PC.SetViewTarget(Bases[HQTeam[T]]);
    		if (!bGameEnded)
    			PC.ClientRoundEnded();
    	}
    	if (!bGameEnded)
	    	C.RoundHasEnded();
    }
    ResetCountDown = ResetTimeDelay;
}

function CheckScore(PlayerReplicationInfo Scorer)
{
	if (CheckMaxLives(Scorer))
		return;

	if (GameRulesModifiers != None && GameRulesModifiers.CheckScore(Scorer))
		return;

	if (GoalScore != 0 && (Teams[0].Score >= GoalScore || Teams[1].Score >= GoalScore))
		EndGame(Scorer,"teamscorelimit");
}

function bool CheckEndGame(PlayerReplicationInfo Winner, string Reason)
{
	if (Reason ~= "TimeLimit")
	{
		if ( !bOverTimeBroadcast )
		{
			StartupStage = 7;
			PlayStartupMessage();
			bOverTimeBroadcast = true;
		}
		return false;
	}

	return Super.CheckEndGame(Winner, Reason);
}

function SetEndGameFocus(PlayerReplicationInfo Winner)
{
	local Controller P;
	local PlayerController player;

	if ( Winner != None )
	{
		if (Winner.Team.TeamIndex == 0)
			EndGameFocus = Bases[HQTeam[1]];
		else
			EndGameFocus = Bases[HQTeam[0]];
	}
	if ( EndGameFocus != None )
		EndGameFocus.bAlwaysRelevant = true;

	for ( P=Level.ControllerList; P!=None; P=P.nextController )
	{
		player = PlayerController(P);
		if ( Player != None )
		{
			if ( !Player.PlayerReplicationInfo.bOnlySpectator )
				PlayWinMessage(Player, (Player.PlayerReplicationInfo.Team == GameReplicationInfo.Winner));
			Player.ClientSetBehindView(true);
			Player.CameraDist = 10;
			if ( EndGameFocus != None )
			{
				Player.ClientSetViewTarget(EndGameFocus);
				Player.SetViewTarget(EndGameFocus);
			}
			Player.ClientGameEnded();
			if ( CurrentGameProfile != None )
				CurrentGameProfile.bWonMatch = (Player.PlayerReplicationInfo.Team == GameReplicationInfo.Winner);
		}
		P.GameHasEnded();
	}
}

function Killed(Controller Killer, Controller Killed, Pawn KilledPawn, class<DamageType> damageType)
{
	if (PlayerController(Killed) != None && GENPlayerReplicationInfo(Killed.PlayerReplicationInfo) != None)
		GENPlayerReplicationInfo(Killed.PlayerReplicationInfo).OwnerDied();

	Super.Killed(Killer, Killed, KilledPawn, damageType);
}

function bool CriticalPlayer(Controller Other)
{
	local int x;

	if (Other.Pawn == None || Other.PlayerReplicationInfo == None)
		return Super.CriticalPlayer(Other);

	for (x = 0; x < Bases.length; x++)
		if ( (Bases[x].CoreStage == 0 || Bases[x].CoreStage == 2) && Bases[x].DefenderTeamIndex != Other.GetTeamNum()
		     && ((Bases[x].bUnderAttack && Other.PlayerReplicationInfo == Bases[x].LastDamagedBy) || VSize(Other.Pawn.Location - Bases[x].Location) < 2000) )
			return true;

	return Super.CriticalPlayer(Other);
}

function bool ValidSpawnPoint(GENPowerCore PC, int TeamIndex)
{
    return (PC != None && PC.DefenderTeamIndex == TeamIndex &&
            !PC.IsInState('NeutralCore') && !PC.IsInState('Reconstruction') &&
            (!PC.bUnderAttack || PC.bFinalCore));
}

function NavigationPoint FindPlayerStart( Controller Player, optional byte InTeam, optional string incomingName )
{
    local NavigationPoint N, BestStart;
    local Teleporter Tel;
    local float BestRating, NewRating;
    local byte Team, EnemyTeam;
    local GENPowerCore SelectedPC;
    local float CoreDistA, CoreDistB;
    local int i;
    local Bot B;

    if ( GameRulesModifiers != None )
    {
        N = GameRulesModifiers.FindPlayerStart(Player,InTeam,incomingName);
        if ( N != None )
            return N;
    }

    // if incoming start is specified, then just use it
    if( incomingName!="" )
        foreach AllActors( class 'Teleporter', Tel )
            if( string(Tel.Tag)~=incomingName )
                return Tel;

    // use InTeam if player doesn't have a team yet
    if ( (Player != None) && (Player.PlayerReplicationInfo != None) )
    {
        if ( Player.PlayerReplicationInfo.Team != None )
            Team = Player.PlayerReplicationInfo.Team.TeamIndex;
        else
            Team = InTeam;
        //Use the powercore the player selected (if it's valid)
		if ( GENPlayerReplicationInfo(Player.PlayerReplicationInfo) != None )
        {
    		SelectedPC = GENPlayerReplicationInfo(Player.PlayerReplicationInfo).TemporaryStartCore;
    		GENPlayerReplicationInfo(Player.PlayerReplicationInfo).TemporaryStartCore = None;
    		if (SelectedPC != None && !ValidSpawnPoint(SelectedPC, Team))
    			SelectedPC = None;
    		if (SelectedPC == None)
    		{
	        	SelectedPC = GENPlayerReplicationInfo(Player.PlayerReplicationInfo).StartCore;
		        if (!ValidSpawnPoint(SelectedPC, Team))
                    SelectedPC = None;
	        }
        }
        if ( SelectedPC == None )
        {
	        //Bots always want to go to their SquadObjective
	        B = Bot(Player);
        	if (B != None && B.Squad != None && GENPowerCore(B.Squad.SquadObjective) != None && ValidSpawnPoint(GENPowerCore(B.Squad.SquadObjective), Team))
        		SelectedPC = GENPowerCore(B.Squad.SquadObjective);
        }
    }
    else
        Team = InTeam;

    if (SelectedPC == None)
    {
    	if (Bases[HQTeam[Team]].PoweredBy(1 - Team))
    		SelectedPC = Bases[HQTeam[Team]];

    	if (SelectedPC == None)
    	{
	        // Find the Closest Controlled Node(s) to Enemy PowerCore.
	        EnemyTeam = abs(Team - 1);
	        BestRating = 255;
	        for (i = 0; i < Bases.Length; i++)
	        {
	            if (ValidSpawnPoint(Bases[i], Team))
	            {
	                NewRating = Bases[i].FinalCoreDistance[EnemyTeam];
	                if ( NewRating < BestRating )
	                {
	                    BestRating = NewRating;
	                    SelectedPC = Bases[i];
	                }
	                else if ( NewRating == BestRating ) // If we have two nodes at equal link distance, we check geometric distance
	                {
                        CoreDistA = VSize(Bases[HQTeam[EnemyTeam]].Location - Bases[i].Location);
                        CoreDistB = VSize(Bases[HQTeam[EnemyTeam]].Location - SelectedPC.Location);
                        if (CoreDistA < CoreDistB)
                            SelectedPC = Bases[i];
                    }
	            }
	        }

	        // If no valid power node found, set to power core.
            if (SelectedPC == None)
		        SelectedPC = Bases[HQTeam[Team]];
        }
    }

    for (i = 0; i < SelectedPC.CloseActors.length; i++)
        if (NavigationPoint(SelectedPC.CloseActors[i]) != None)
        {
	        NewRating = RatePlayerStart(NavigationPoint(SelectedPC.CloseActors[i]),Team,Player);
        	if ( NewRating > BestRating )
	        {
        	    BestRating = NewRating;
	            BestStart = NavigationPoint(SelectedPC.CloseActors[i]);
        	}
	}

    if (PlayerStart(BestStart) == None && SelectedPC != Bases[HQTeam[Team]])
    {
    	//couldn't find a start at the requested node, so try powercore
    	SelectedPC = Bases[HQTeam[Team]];
    	for (i = 0; i < SelectedPC.CloseActors.length; i++)
	        if (NavigationPoint(SelectedPC.CloseActors[i]) != None)
	        {
		        NewRating = RatePlayerStart(NavigationPoint(SelectedPC.CloseActors[i]),Team,Player);
	        	if ( NewRating > BestRating )
		        {
	        	    BestRating = NewRating;
		            BestStart = NavigationPoint(SelectedPC.CloseActors[i]);
	        	}
		}
    }

    if (PlayerStart(BestStart) == None)
    {
        log("Warning - PATHS NOT DEFINED or NO PLAYERSTART with positive rating");
	BestRating = -100000000;
        ForEach AllActors( class 'NavigationPoint', N )
        {
            NewRating = RatePlayerStart(N,0,Player);
            if ( InventorySpot(N) != None )
				NewRating -= 50;
			NewRating += 20 * FRand();
            if ( NewRating > BestRating )
            {
                BestRating = NewRating;
                BestStart = N;
            }
        }
    }

    return BestStart;
}

function ShowPathTo(PlayerController P, int TeamNum)
{
	local int x;
	local GENPowerCore Best;
	local float BestDist;
	local class<WillowWhisp>	WWclass;

	for (x = 0; x < Bases.Length; x++)
		if ( (Bases[x].CoreStage != 0 || Bases[x].DefenderTeamIndex != TeamNum) && Bases[x].PoweredBy(TeamNum)
		     && (Best == None || VSize(P.Pawn.Location - Bases[x].Location) < BestDist) )
		{
			Best = Bases[x];
			BestDist = VSize(P.Pawn.Location - Bases[x].Location);
		}

	if (Best != None && P.FindPathToward(Best, false) != None)
	{
		WWclass = class<WillowWhisp>(DynamicLoadObject(PathWhisps[TeamNum], class'Class'));
		Spawn(WWclass, P,, P.Pawn.Location);
	}
}

function GetServerInfo(out ServerResponseLine ServerState)
{
	Super.GetServerInfo(ServerState);

	if (!bCurrentSetupIsOfficial)
		ServerState.MapName $= "*";
}

function GetServerDetails(out ServerResponseLine ServerState)
{
	Super.GetServerDetails(ServerState);

	AddServerDetail( ServerState, "LinkSetup", CurrentSetupName );
}

function bool ChangeTeam(Controller Other, int num, bool bNewTeam)
{
	local int i;
	local GENPlayerReplicationInfo PRI;

	if (Super.ChangeTeam(Other, num, bNewTeam))
	{
		if (Other == Level.GetLocalPlayerController())
		{
			//Update client side effects on Bases to reflect which ones the player can go after changing teams
			for (i = 0; i < Bases.length; i++)
				Bases[i].CheckShield();
		}

		PRI = GENPlayerReplicationInfo(Other.PlayerReplicationInfo);
		if (PRI != None)
		{
			PRI.StartCore = None;
			PRI.TemporaryStartCore = None;
			PRI.resetRes();
		}

		return true;
	}

	return false;
}

function PlayStartupMessage()
{
	Super.PlayStartupMessage();

	if (StartupStage == 7 && OvertimeCoreDrainPerSec > 0)
		BroadcastLocalizedMessage(class'GENGenerationsMessage', 29);
}

function ScoreObjective(PlayerReplicationInfo Scorer, float Score)
{
	if (Scorer != None)
		Scorer.Score += Score;

	if (GameRulesModifiers != None)
		GameRulesModifiers.ScoreObjective(Scorer,Score);
}

static event bool AcceptPlayInfoProperty(string PropertyName)
{
	if ( InStr(PropertyName, "bAllowTrans") != -1 )
		return false;

	return Super.AcceptPlayInfoProperty(PropertyName);
}

static function FillPlayInfo(PlayInfo PlayInfo)
{
	local int i;

	Super.FillPlayInfo(PlayInfo);  // Always begin with calling parent

	PlayInfo.AddSetting(default.GameGroup, "OvertimeCoreDrainPerSec", default.GENPropsDisplayText[i++], 40, 1, "Text", "2;0:99");
	PlayInfo.AddSetting(default.GameGroup, "bRandSetupAfterReset",    default.GENPropsDisplayText[i++], 0,  1, "Check",        ,,     , true);
	PlayInfo.AddSetting(default.GameGroup, "bSwapSidesAfterReset",    default.GENPropsDisplayText[i++], 0,  1, "Check",        ,,     , true);
	PlayInfo.AddSetting(default.GameGroup, "bUseCustomModels",    default.GENPropsDisplayText[i++], 0,  1, "Check",        ,,     , false);
}

static event string GetDescriptionText(string PropName)
{
	switch (PropName)
	{
		case "OvertimeCoreDrainPerSec":	return default.GENPropDescText[0];
		case "bRandSetupAfterReset":	return default.GENPropDescText[1];
		case "bSwapSidesAfterReset":	return default.GENPropDescText[2];
		case "bUseCustomModels":	return default.GENPropDescText[3];
	}

	return Super.GetDescriptionText(PropName);
}

static function array<string> GetAllLoadHints(optional bool bThisClassOnly)
{
	local int i;
	local array<string> Hints;

	if ( !bThisClassOnly || default.GENHints.Length == 0 )
		Hints = Super.GetAllLoadHints();

	for ( i = 0; i < default.GENHints.Length; i++ )
		Hints[Hints.Length] = default.GENHints[i];

	return Hints;
}

event Destroyed()
{
	CustomPowerLinkSetups.length = 0;

	Super.Destroyed();
}

function SetupLinks()
{
	local GENPowerCore Core;
	local int i;

	if ( Bases.Length == 0 )
		return;

	Core = Bases[0];
	do
	{
		for ( i = 0; i < Core.PowerLinks.Length; i++ )
			AddLink(Core, Core.PowerLinks[i]);

		Core = Core.NextCore;
	} until ( Core == None || Core == Bases[0] );

}

function bool AddLink( GENPowerCore A, GENPowerCore B )
{
	local GENPowerLink Link;

	if ( HasLink(A,B) )
		return false;

	Link = Spawn(class'GENPowerLink');
	Link.SetNodes(A,B);

//	log(Name@"AddLink Link:"$Link.Name@"A:"$A.Name@"B:"$B.Name);
	PowerLinks[PowerLinks.Length] = Link;
//	A.AddPowerLink(B);
//	A.PowerCoreReset();
//	B.PowerCoreReset();
	return True;
}

function bool RemoveLink( GENPowerCore A, GENPowerCore B )
{
	local int i;

	for ( i = PowerLinks.Length - 1; i >= 0; i-- )
	{
		if ( PowerLinks[i] == None )
		{
			PowerLinks.Remove(i,1);
			continue;
		}

		if ( PowerLinks[i].HasNodes(A,B) )
		{
//			A.RemovePowerLink(B);

//			A.PowerCoreReset();
//			B.PowerCoreReset();
			PowerLinks[i].Destroy();
			PowerLinks.Remove(i,1);
			return True;
		}
	}

	return False;
}

function bool HasLink( GENPowerCore A, GENPowerCore B )
{
	local int i;

	for ( i = PowerLinks.Length - 1; i >= 0; i-- )
	{
		if ( PowerLinks[i] == None )
		{
			PowerLinks.Remove(i,1);
			continue;
		}

		if ( PowerLinks[i].HasNodes(A,B) )
			return True;
	}

	return False;
}

event PostLogin(PlayerController NewPlayer)
{
	Super.PostLogin(NewPlayer);

	if ( GENPlayerReplicationInfo(NewPlayer.PlayerReplicationInfo) != None )
		GENPlayerReplicationInfo(NewPlayer.PlayerReplicationInfo).ClientPrepareToReceivePowerLinks();
}

function DeadUse(PlayerController PC)
{
	Super.DeadUse(PC);
	PC.ShowMenu();
}


defaultproperties
{
    ScreenShotName="Generations.GENThumbnail"
    GameName="Generations"
    MapPrefix="GEN"
    BeaconName="GEN"
    DefaultPlayerClassName="GTDeify.Deify_xPawn"
    PlayerControllerClassName = "GTDeify.Deify_xPlayer"

    ADR_Kill=2.0
    bAllowVehicles=true
    bScoreTeamKills=False
    bSpawnInTeamArea=True
    bTeamScoreRounds=False
    bScoreVictimsTarget=false
    ResetTimeDelay=11
    NetWait=15
    GoalScore=3
    OvertimeCoreDrainPerSec=20
    bSwapSidesAfterReset=True

    MapListType="Generations.GENMapListGenerations"
    HUDType="Generations.GENHUDGenerations"
    DeathMessageClass=class'XGame.xDeathMessage'
    MutatorClass="Generations.GENDefaultMut"

    DecoTextName="Generations.GENGameInfo"

    Acronym="GEN"
    DefaultEnemyRosterClass="xGame.xTeamRoster"
    TeamAIType(0)=class'Generations.GENTeamAI'
    TeamAIType(1)=class'Generations.GENTeamAI'
	bLiberalVehiclePaths=true

    bAlwaysShowLoginMenu=false

    GameUMenuType="Generations.UT2K4GenerationsLoginMenu"
    LoginMenuClass="Generations.UT2K4GenerationsLoginMenu"

    GENPropsDisplayText(0)="HQ decomposition in Overtime"
    GENPropsDisplayText(1)="Random Link Setup After Reset"
    GENPropsDisplayText(2)="Teams Swap Sides After Reset"
    GENPropsDisplayText(3)="Use Custom Character Models - UNSTABLE"
    GENPropDescText(0)="In overtime, Headquarters lose a maximum of this much health every second."
    GENPropDescText(1)="After a reset, a new link setup will be chosen at random."
    GENPropDescText(2)="After a reset, teams will switch sides so they are defending the headquarters they were previously attacking."
    GENPropDescText(3)="Replace default player models with development Generations Berserker models. Runes on the body are coloured according to class. Be aware this causes occasional crashes due to incomplete Karma data."

    Description="Generations - The new age of man. || Choose a class and direct your troops to capture waypoints and gain resources. Buy upgrades with resources and capture a path to the enemy headquarters. || Use your spells to gain the upper hand and lock on to enemies with your sword to claim victory."

    GENHints(0)="Direct your troops by selecting a start and end waypoint in the command menu."
    GENHints(1)="Bind a key to bring up the command menu from the UT input settings."
    GENHints(2)="Only troops can capture waypoints and HQs."
    GENHints(3)="Spend resources to upgrade your passive abilities, magic abilities, or technology upgrades."
    GENHints(4)="Capture vulnerable neutral waypoints by outnumbering enemy troops in the area."
    GENHints(5)="Make good use of the troops, they may not be as good as a champion, but can turn the tide of battle in large numbers."
    GENHints(6)="Upgrade as soon as you can using the buttons on the command menu."
    GENHints(7)="Each of the classes has a different set of upgrade features."
    GENHints(8)="The Berserker nature upgrade increases sword damage."
    GENHints(9)="Charge up your spells for a longer-lasting effect."

	bUseCustomModels = false
}
