class GENPlayerReplicationInfo extends xPlayerReplicationInfo
	config(user);

var int resources;
var int tech[3];
var int cost[3];
var string techtree[3];
var string techupgrade[9];

var GENPowerCore StartCore; //always try to spawn player here. None means auto-select, value ignored if TemporaryStartCore != None
var GENPowerCore TemporaryStartCore; //next time player respawns try to put him here (then this value is cleared)

enum EMapPreference
{
	MAP_Never,
	MAP_WhenBodyStill,
	MAP_Immediately
};

var globalconfig EMapPreference ShowMapOnDeath;
var bool bPendingMapDisplay;
var globalconfig string LinkDesignerClass;	//class of Link Designer menu
var Material TransMaterials[2];
var float PendingHealBonus; //node healing score bonus that hasn't yet been added to Score

replication
{
	reliable if (bNetDirty && Role == ROLE_Authority)
		StartCore, resources, tech;
	reliable if (Role < ROLE_Authority)
		cycleBotDest, setBotDest, SetStartCore, ServerSendPowerLinks, ServerReceivePowerLink, ServerRemovePowerLink, ServerClearPowerLinks, ServerSendSetupNames,
		ServerSetPowerLinkSetup, ServerSavePowerLinkSetup, ServerDeletePowerLinkSetup, RequestLinkDesigner, TeleportTo, upgrade;
	reliable if (Role == ROLE_Authority)
		ClientPrepareToReceivePowerLinks, ClientReceivePowerLink, ClientRemovePowerLink, ClientReceiveSetupName,
		ClientReceiveCurrentSetup, ClientShowMap;
}

//these are set by the link designer
delegate OnReceiveLink( GENPowerCore A, GENPowerCore B );
delegate OnRemoveLink( GENPowerCore A, GENPowerCore B );
delegate ResetLinks();
delegate ProcessSetupName(string SetupName);
delegate RemoveSetupName(string SetupName);
delegate SetCurrentSetup(string SetupName);
delegate UpdateGUIMapLists(); //listen/standalone servers only

function cycleBotDest(GENPowerCore core)
{
	if (Role == ROLE_Authority)
		core.cycleBotDest();
}

function setBotDest(GENPowerCore core,GENPowerCore destination)
{
	if (Role == ROLE_Authority)
		core.setBotDest(destination);
}

function SetStartCore(GENPowerCore Core, bool bTemporary)
{
	if ( Core == None || GENGameInfo(Level.Game).ValidSpawnPoint(Core, Team.TeamIndex))
	{
		if (bTemporary)
			TemporaryStartCore = Core;
		else
			StartCore = Core;
	}
}

function resetRes()
{
	resources = 0;
	tech[0] = 0;
	tech[1] = 0;
	tech[2] = 0;
}

// returns the powecore/node the player is currently standing on or in the node teleport trigger radius of,
// if that core/node is currently constructed for the same team as the player
simulated function GENPowerCore GetCurrentNode()
{
	local GENPowerCore Core;
	local GENPCTeleportTrigger T;

	if (Controller(Owner).Pawn != None)
	{
		if (Controller(Owner).Pawn.Base != None)
		{
			Core = GENPowerCore(Controller(Owner).Pawn.Base);
			if (Core == None)
				Core = GENPowerCore(Controller(Owner).Pawn.Base.Owner);
		}

		if (Core == None)
			foreach Controller(Owner).Pawn.TouchingActors(class'GENPCTeleportTrigger', T)
			{
				Core = GENPowerCore(T.Owner);
				if (Core != None)
					break;
			}
	}

	if (Core != None && Core.CoreStage == 0 && Core.DefenderTeamIndex == Team.TeamIndex)
		return Core;

	return None;
}

function TeleportTo(GENPowerCore Core)
{
	local GENPowerCore OldStartCore;
	local GENPowerCore OwnerBase;

	OwnerBase = GetCurrentNode();
	if (OwnerBase != None && GENGameInfo(Level.Game).ValidSpawnPoint(Core, Team.TeamIndex))
	{
		OldStartCore = StartCore;
		StartCore = Core;
		if (!DoTeleport() && !DoTeleport()) //two tries
		{
			//FIXME error localmessage here!
		}
		StartCore = OldStartCore;
	}
}

function bool DoTeleport()
{
	local NavigationPoint NewStart;
	local vector PrevLocation;

	PrevLocation = Controller(Owner).Pawn.Location;
	NewStart = Level.Game.FindPlayerStart(Controller(Owner));
	if (NewStart != None && Controller(Owner).Pawn.SetLocation(NewStart.Location))
	{
		Controller(Owner).ClientSetRotation(NewStart.Rotation);
		if (xPawn(Controller(Owner).Pawn) != None)
			xPawn(Controller(Owner).Pawn).DoTranslocateOut(PrevLocation);
		Controller(Owner).Pawn.SetOverlayMaterial(TransMaterials[Team.TeamIndex], 1.0, false);
		Controller(Owner).Pawn.PlayTeleportEffect(false, false);
		return true;
	}

	return false;
}

function OwnerDied()
{
	if (PlayerController(Owner) != None )
		ClientShowMap();
}

simulated function ClientShowMap()
{
	if (ShowMapOnDeath > MAP_Never)
	{
		bPendingMapDisplay = true;
		SetTimer(0.75, true);
	}
}

simulated event Timer()
{
	local PlayerController PC;

	if (bPendingMapDisplay)
	{
		PC = Level.GetLocalPlayerController();
		if (PC != None && !PC.bDemoOwner && PC.Player != None && GUIController(PC.Player.GUIController) != None && !GUIController(PC.Player.GUIController).bActive)
		{
			if (PC.Pawn != None && PC.Pawn.Health > 0) //never automatically pop up map while player is playing!
			{
				bPendingMapDisplay = false;
				SetTimer(0, false);
			}
			else if (ShowMapOnDeath == MAP_Immediately || PC.ViewTarget == None || PC.ViewTarget == PC || VSize(PC.ViewTarget.Velocity) < 100)
			{
				PC.bMenuBeforeRespawn = false;
				bPendingMapDisplay = false;
				SetTimer(0, false);
				PC.ShowMidGameMenu(false);
			}
			else
				PC.bMenuBeforeRespawn = true;
		}
		else
		{
			bPendingMapDisplay = False;
			SetTimer(0, false);
		}
	}

	Super.Timer();
}

simulated function ClientPrepareToReceivePowerLinks()
{
	ResetLinks();
}

function ServerSendPowerLinks()
{
	local int i;
	local GENGameInfo Game;

	Game = GENGameInfo(Level.Game);
	for ( i = 0; i < Game.PowerLinks.Length; i++ )
		ClientReceivePowerLink(Game.PowerLinks[i].Nodes[0].Node, Game.PowerLinks[i].Nodes[1].Node);
}

simulated function ClientReceivePowerLink(GENPowerCore From, GENPowerCore To)
{
//	log(Name@"ClientReceivePowerLink A:"$From.Name@"B:"$To.Name);
	OnReceiveLink(From,To);
}

simulated function ClientRemovePowerLink(GENPowerCore From, GENPowerCore To)
{
	OnRemoveLink(From,To);
}

function ServerReceivePowerLink(GENPowerCore From, GENPowerCore To)
{
	local int i;
	local GameReplicationInfo GRI;

	if (PlayerController(Owner) == None || (Owner != Level.GetLocalPlayerController() && !bAdmin) || From == To)
		return;

	if ( GENGameInfo(Level.Game).AddLink(From,To) )
	{
		GENGameInfo(Level.Game).UpdateLinks();
		GRI = Level.Game.GameReplicationInfo;
		for ( i = 0; i < GRI.PRIArray.Length; i++ )
		{
			if ( PlayerController(GRI.PRIArray[i].Owner) == None ||
				 GENPlayerReplicationInfo(GRI.PRIArray[i]) == None )
				continue;

			GENPlayerReplicationInfo(GRI.PRIArray[i]).ClientReceivePowerLink(From,To);
		}
	}

}

function ServerRemovePowerLink(GENPowerCore From, GENPowerCore To)
{
	local int i;
	local GameReplicationInfo GRI;

	if (PlayerController(Owner) == None || (Owner != Level.GetLocalPlayerController() && !bAdmin) || From == To)
		return;

	if ( GENGameInfo(Level.Game).RemoveLink(From,To) )
	{
		GENGameInfo(Level.Game).UpdateLinks();
		GRI = Level.Game.GameReplicationInfo;
		for ( i = 0; i < GRI.PRIArray.Length; i++ )
		{
			if ( PlayerController(GRI.PRIArray[i].Owner) == None ||
				 GENPlayerReplicationInfo(GRI.PRIArray[i]) == None )
				continue;

			GENPlayerReplicationInfo(GRI.PRIArray[i]).ClientRemovePowerLink(From,To);
		}
	}
}

function ServerClearPowerLinks()
{
	local int i;
	local GENGameInfo Game;

	Game = GENGameInfo(Level.Game);

	while (Game.PowerLinks.length > 0)
		ServerRemovePowerLink(Game.PowerLinks[i].Nodes[0].Node, Game.PowerLinks[i].Nodes[1].Node);
}

function ServerSendSetupNames()
{
	local int x;

	for (x = 0; x < GENGameInfo(Level.Game).OfficialPowerLinkSetups.length && x < 1000; x++)
		ClientReceiveSetupName(GENGameInfo(Level.Game).OfficialPowerLinkSetups[x].SetupName);

	for (x = 0; x < GENGameInfo(Level.Game).CustomPowerLinkSetups.length && x < 1000; x++)
		ClientReceiveSetupName(string(GENGameInfo(Level.Game).CustomPowerLinkSetups[x].Name));

	ClientReceiveSetupName("");		//Indicate end of setup names

	ClientReceiveCurrentSetup(GENGameInfo(Level.Game).CurrentSetupName);
}

simulated function ClientReceiveSetupName(string SetupName)
{
	ProcessSetupName(SetupName);
}

simulated function ClientReceiveCurrentSetup(string SetupName)
{
	SetCurrentSetup(SetupName);
}

function ServerSetPowerLinkSetup(string SetupName)
{
	if (PlayerController(Owner) == None || (Owner != Level.GetLocalPlayerController() && !bAdmin))
		return;

	GENGameInfo(Level.Game).SetPowerLinkSetup(SetupName, PlayerController(Owner));
}

function ServerSavePowerLinkSetup(string SetupName)
{
	local array<GENPowerCore> Traveled;

	if (PlayerController(Owner) == None || (Owner != Level.GetLocalPlayerController() && !bAdmin))
		return;

	if ( !GENGameInfo(Level.Game).ExistsLinkPathTo( GENGameInfo(Level.Game).Bases[GENGameInfo(Level.Game).HQTeam[0]],
							     GENGameInfo(Level.Game).Bases[GENGameInfo(Level.Game).HQTeam[1]],
							     Traveled ) )
	{
		PlayerController(Owner).ReceiveLocalizedMessage(class'ONSLinkDesignMessage', 7);
		return;
	}

	if (GENGameInfo(Level.Game).SavePowerLinkSetup(SetupName, PlayerController(Owner)))
	{
		ClientReceiveSetupName(SetupName);
		UpdateGUIMapLists();
	}
}

function ServerDeletePowerLinkSetup(string SetupName)
{
	if (PlayerController(Owner) == None || (Owner != Level.GetLocalPlayerController() && !bAdmin))
		return;

	if (GENGameInfo(Level.Game).DeletePowerLinkSetup(SetupName, PlayerController(Owner)))
	{
		ClientRemoveSetupName(SetupName);
		UpdateGUIMapLists();
	}
}

simulated function ClientRemoveSetupName(string SetupName)
{
	RemoveSetupName(SetupName);
}

function RequestLinkDesigner()
{
	if (PlayerController(Owner) != None && (Owner == Level.GetLocalPlayerController() || bAdmin))
		PlayerController(Owner).ClientOpenMenu(LinkDesignerClass);
}

//scoring bonus for healing powernodes
//we wait until the healing bonus >= 1 point before adding to score to minimize ScoreEvent() calls
function AddHealBonus(float Bonus)
{
	PendingHealBonus += Bonus;
	if (PendingHealBonus >= 1.f)
	{
		Level.Game.ScoreObjective(self, PendingHealBonus);
		Level.Game.ScoreEvent(self, PendingHealBonus, "heal_powernode");
		PendingHealBonus = 0;
	}
}

function Reset()
{
	HasFlag = None;
	NumLives = 0;
	bOutOfLives = false;
	resetRes();
}

simulated function upgrade(int treeNumber)
{
	if ( Role == ROLE_Authority ) {
    if (treeNumber < 3 && tech[treeNumber] < 3 && cost[tech[treeNumber]] <= resources)
	{
		resources -= cost[tech[treeNumber]];
		tech[treeNumber]++;
		upgradePawn();
	}}
}

function upgradePawn()
{
	if (GENUpgradeableXPawn(Controller(Owner).Pawn) != none)
	{
		GENUpgradeableXPawn(Controller(Owner).Pawn).upgrade(0,tech[0]);
		GENUpgradeableXPawn(Controller(Owner).Pawn).upgrade(1,tech[1]);
		GENUpgradeableXPawn(Controller(Owner).Pawn).upgrade(2,tech[2]);
	}
}

function checkWeapons()
{
    local int i;
    local int j;

    for(i = 0; i < 3; i++)
    {
        for(j = 0; j < tech[i]; j++)
        {
            Controller(Owner).Pawn.GiveWeapon(techupgrade[i * 3 + j]);
        }
    }
}

defaultproperties
{
	resources=0
	tech[0]=0
	tech[1]=0
	tech[2]=0
	cost[0]=50
	cost[1]=150
	cost[2]=400
	techtree[0]="Melee"
	techtree[1]="Projectile"
	techtree[2]="Beam"
	techupgrade[0]="XWeapons.ShieldGun"
	techupgrade[1]="XWeapons.ShieldGun"
	techupgrade[2]="XWeapons.ShieldGun"
	techupgrade[3]="XWeapons.AssaultRifle"
	techupgrade[4]="XWeapons.Minigun"
	techupgrade[5]="XWeapons.RocketLauncher"
	techupgrade[6]="XWeapons.ShockRifle"
	techupgrade[7]="XWeapons.LinkGun"
	techupgrade[8]="XWeapons.SniperRifle"

	ShowMapOnDeath=MAP_WhenBodyStill
	LinkDesignerClass="Generations.GENPowerLinkDesigner"
	TransMaterials(0)=Material'PlayerTransRed'
	TransMaterials(1)=Material'PlayerTrans'
}
