// ====================================================================
// Generations command menu
// Written by Daniel Bower
// ====================================================================

#EXEC OBJ LOAD FILE=Generations.utx

class UT2K4GENCommandMenu extends Interaction;

var() bool bCaptureAxis;
var int cursorX;
var int cursorY;
var int resX;
var int resY;
var UT2K4GENUpgradeButton upgradeButton[3];

event Initialized()
{
	Super.Initialized();
	resX = ViewportOwner.Actor.myHud.ResScaleX * 640;
	resY = ViewportOwner.Actor.myHud.ResScaleY * 480;
	cursorX = resX * 0.5;
	cursorY = resY * 0.5;
	upgradeButton[0] = new class'UT2K4GENUpgradeButton';
	upgradeButton[0].Init(0,resX * 0.62,resY * 0.3, resX * 0.15, resY * 0.1 ,ViewportOwner.Actor.myHud);
	upgradeButton[1] = new class'UT2K4GENUpgradeButton';
	upgradeButton[1].Init(1,resX * 0.62,resY * 0.43, resX * 0.15, resY * 0.1 ,ViewportOwner.Actor.myHud);
	upgradeButton[2] = new class'UT2K4GENUpgradeButton';
	upgradeButton[2].Init(2,resX * 0.62,resY * 0.56, resX * 0.15, resY * 0.1 ,ViewportOwner.Actor.myHud);
}

function PostRender( Canvas C )
{
	C.SetPos(resX * 0.2 , resY * 0.2 );
	if (ViewportOwner.Actor.myHud.PawnOwnerPRI.Team.TeamIndex == 0)
		C.SetDrawColor(255,0,0);
	else
		C.SetDrawColor(0,0,255);
	C.DrawTileStretched(Texture'Generations.command_back', resX * 0.6, resY * 0.5);

	GENHUDGenerations(ViewportOwner.Actor.myHud).DrawRadarMap(C ,resX * 0.4, resY * 0.45, resX * 0.18, false);

	upgradeButton[0].Render(C);
	upgradeButton[1].Render(C);
	upgradeButton[2].Render(C);

	C.SetDrawColor(255,255,255);
	C.SetPos(cursorX, cursorY);
	C.DrawIcon(Texture'Generations.cursor', 1.0);
}

function bool KeyEvent(EInputKey Key, EInputAction Action, FLOAT Delta )
{
    local UT2K4GENUpgradeButton button;

    if (Action == IST_Press && (Key == IK_LeftMouse || Key == IK_RightMouse))
    {
    	button = hoveredButton();
    	if (button != none && Key == IK_LeftMouse)
    	{
    		button.bSelected = true;
    		GENHudGenerations(ViewportOwner.Actor.myHUD).setCoreBuffer = none;
    	}
    	else
    		GENHudGenerations(ViewportOwner.Actor.myHUD).botOrders(cursorX - resX * 0.4, cursorY - resY * 0.45, resX * 0.18);
    		
    	return true;
    }
    else if (Action == IST_Release && Key == IK_LeftMouse)
    {
        button = hoveredButton();
        if (button != none && button.bSelected)
    		button.clicked();
    	upgradeButton[0].bSelected = false;
    	upgradeButton[1].bSelected = false;
    	upgradeButton[2].bSelected = false;
        return false; //Allow click releases to be registered so players don't keep firing.
    }
    else if (Key == IK_RightMouse && Action == IST_Press)
    	return true; //ignore secondary while we're at it.
    else if (Action == IST_Axis && Key == IK_MouseX)
    {
        cursorX = Clamp(cursorX + Delta,0,resX);
        return bCaptureAxis;
    }
    else if (Action == IST_Axis && Key == IK_MouseY)
    {
    	cursorY = Clamp(cursorY - Delta,0,resY);
    	return bCaptureAxis;
    }


    return false;
}

function UT2K4GENUpgradeButton hoveredButton()
{
	local int i;

	for (i = 0; i < 3; i++)
	{
		if (cursorX > upgradeButton[i].posX && cursorX < upgradeButton[i].posX + upgradeButton[i].width
    	&& cursorY > upgradeButton[i].posY && cursorY < upgradeButton[i].posY + upgradeButton[i].height)
    		return upgradeButton[i];
    }
    return none;
}

event NotifyLevelChange()
{
	Cleanup();
	Master.RemoveInteraction(self);
}

function show()
{
	cursorX = resX * 0.5;
	cursorY = resY * 0.5;
	bActive = true;
	bVisible = true;
}

function hide()
{
	bActive = false;
	bVisible = false;
	GENHudGenerations(ViewportOwner.Actor.myHUD).setCoreBuffer = none;
	upgradeButton[0].bSelected = false;
	upgradeButton[1].bSelected = false;
	upgradeButton[2].bSelected = false;
}

function Cleanup()
{
	upgradeButton[0].Cleanup();
	upgradeButton[1].Cleanup();
	upgradeButton[2].Cleanup();
}

defaultproperties
{
	bCaptureAxis = true
	resX = 640
	resY = 480
	cursorX=0
	cursorY=0
	bActive=false
	bVisible=false
}
